module PolygonBug exposing (..)

import BoundingBox2d
import Color
import Drawing2d
import Drawing2d.Attributes as Attributes
import Drawing2d.Text as Text
import Html exposing (Html)
import Point2d exposing (Point2d)
import Polygon2d exposing (Polygon2d)
import Rectangle2d
import Triangle2d
import TriangularMesh


drawVertex : Int -> Point2d -> Drawing2d.Element msg
drawVertex index point =
    Drawing2d.group
        [ Drawing2d.dot point
        , Drawing2d.textWith [ Attributes.textAnchor Text.center, Attributes.fontSize 16 ]
            point
            (toString index)
        ]


main : Html msg
main =
    let
        renderBounds =
            BoundingBox2d.fromExtrema
                { minX = 0
                , maxX = 550
                , minY = -10
                , maxY = 760
                }

        zoomRectangle =
            Rectangle2d.fromExtrema
                { minX = 135
                , maxX = 145
                , minY = 685
                , maxY = 690
                }

        zoomFactor =
            150

        centerPoint =
            Rectangle2d.centerPoint zoomRectangle

        zoomedPolygon =
            Polygon2d.scaleAbout centerPoint zoomFactor testPolygon

        zoomedBounds =
            Rectangle2d.scaleAbout centerPoint zoomFactor zoomRectangle
                |> Rectangle2d.boundingBox

        triangles =
            Polygon2d.triangulate testPolygon
                |> TriangularMesh.faceVertices
                |> List.map Triangle2d.fromVertices

        _ =
            Debug.log "number of triangles" (List.length triangles)
    in
    Drawing2d.toHtml zoomedBounds [] <|
        [ Drawing2d.polygon zoomedPolygon
        , Drawing2d.group
            (List.indexedMap drawVertex (Polygon2d.vertices zoomedPolygon))
        ]



--Drawing2d.toHtml renderBounds [] <|
--    [ Drawing2d.groupWith [ Attributes.strokeColor Color.darkGrey ]
--        (List.map Drawing2d.triangle triangles)
--    , Drawing2d.polygonWith [ Attributes.noFill ] testPolygon
--    ]


testPolygon : Polygon2d
testPolygon =
    let
        vertexCoordinates =
            [ ( 356, 750 )
            , ( 354.29800000000006, 738.909 )
            , ( 350.66400000000004, 702.552 )
            , ( 345.506, 646.203 )
            , ( 339.232, 575.136 )
            , ( 332.25, 494.625 )
            , ( 324.968, 409.944 )
            , ( 317.794, 326.36699999999996 )
            , ( 311.13599999999997, 249.168 )
            , ( 305.402, 183.621 )
            , ( 301, 135 )
            , ( 298, 134 )
            , ( 291.63399999999996, 242.67199999999997 )
            , ( 284.872, 342.65599999999995 )
            , ( 277.318, 433.30400000000003 )
            , ( 268.576, 513.9680000000001 )
            , ( 258.25, 584 )
            , ( 245.944, 642.752 )
            , ( 231.262, 689.576 )
            , ( 213.808, 723.8240000000001 )
            , ( 193.18599999999998, 744.8480000000001 )
            , ( 169, 752 )
            , ( 158.489, 752 )
            , ( 148.512, 752 )
            , ( 139.003, 752 )
            , ( 129.896, 752 )
            , ( 121.125, 752 )
            , ( 112.624, 752 )
            , ( 104.327, 752 )
            , ( 96.168, 752 )
            , ( 88.081, 752 )
            , ( 80, 752 )
            , ( 80.056, 745.1170000000001 )
            , ( 80.208, 727.296 )
            , ( 80.43199999999999, 700.0790000000001 )
            , ( 80.704, 665.0079999999999 )
            , ( 81, 623.625 )
            , ( 81.296, 577.472 )
            , ( 81.56800000000001, 528.091 )
            , ( 81.792, 477.024 )
            , ( 81.944, 425.813 )
            , ( 82, 376 )
            , ( 81.439, 317.224 )
            , ( 79.912, 261.35200000000003 )
            , ( 77.653, 209.06799999999998 )
            , ( 74.896, 161.056 )
            , ( 71.875, 118 )
            , ( 68.824, 80.584 )
            , ( 65.977, 49.492000000000004 )
            , ( 63.568000000000005, 25.408 )
            , ( 61.831, 9.016000000000002 )
            , ( 61, 1 )
            , ( 66.437, 0.9239999999999999 )
            , ( 73.416, 0.7520000000000001 )
            , ( 81.43900000000001, 0.568 )
            , ( 90.008, 0.45599999999999996 )
            , ( 98.625, 0.5 )
            , ( 106.792, 0.7839999999999999 )
            , ( 114.011, 1.3920000000000003 )
            , ( 119.784, 2.4080000000000004 )
            , ( 123.613, 3.916 )
            , ( 125, 6 )
            , ( 126.018, 20.684999999999995 )
            , ( 127.744, 41.63999999999999 )
            , ( 129.986, 68.41500000000002 )
            , ( 132.552, 100.56 )
            , ( 135.25, 137.625 )
            , ( 137.888, 179.16 )
            , ( 140.274, 224.715 )
            , ( 142.21599999999998, 273.84000000000003 )
            , ( 143.52200000000002, 326.08500000000004 )
            , ( 144, 381 )
            , ( 143.78, 414.301 )
            , ( 143.20000000000002, 449.40799999999996 )
            , ( 142.38, 485.427 )
            , ( 141.44, 521.464 )
            , ( 140.5, 556.625 )
            , ( 139.68, 590.0160000000001 )
            , ( 139.10000000000002, 620.743 )
            , ( 138.88, 647.912 )
            , ( 139.14, 670.629 )

            --, ( 140, 688 )
            --, ( 140.459, 687.946 )
            --, ( 140.192, 687.808 )
            --, ( 139.433, 687.622 )
            --, ( 138.416, 687.424 )
            --, ( 137.375, 687.25 )
            --, ( 136.544, 687.136 )
            --, ( 136.157, 687.1179999999999 )
            --, ( 136.44799999999998, 687.232 )
            --, ( 137.65099999999998, 687.514 )
            , ( 140, 688 )
            , ( 157.796, 676.025 )
            , ( 173.528, 635.28 )
            , ( 187.412, 572.035 )
            , ( 199.664, 492.55999999999995 )
            , ( 210.5, 403.125 )
            , ( 220.136, 310 )
            , ( 228.78799999999998, 219.45499999999998 )
            , ( 236.672, 137.76000000000002 )
            , ( 244.004, 71.185 )
            , ( 251, 26 )
            , ( 252.507, 20.814 )
            , ( 254.656, 16.072000000000003 )
            , ( 257.489, 11.797999999999998 )
            , ( 261.048, 8.015999999999998 )
            , ( 265.375, 4.75 )
            , ( 270.512, 2.0240000000000005 )
            , ( 276.501, -0.1380000000000008 )
            , ( 283.384, -1.712 )
            , ( 291.203, -2.674 )
            , ( 300, -3 )
            , ( 304.86699999999996, -2.745 )
            , ( 309.296, -1.9200000000000004 )
            , ( 313.329, -0.4349999999999994 )
            , ( 317.008, 1.8000000000000007 )
            , ( 320.375, 4.875 )
            , ( 323.472, 8.88 )
            , ( 326.341, 13.905000000000001 )
            , ( 329.024, 20.04 )
            , ( 331.563, 27.374999999999996 )
            , ( 334, 36 )
            , ( 339.348, 59.49699999999999 )
            , ( 344.744, 90.29599999999999 )
            , ( 350.416, 129.15900000000002 )
            , ( 356.59200000000004, 176.848 )
            , ( 363.5, 234.125 )
            , ( 371.368, 301.752 )
            , ( 380.424, 380.491 )
            , ( 390.89599999999996, 471.104 )
            , ( 403.012, 574.353 )
            , ( 417, 691 )
            , ( 417.757, 691.0709999999999 )
            , ( 419.816, 691.208 )
            , ( 422.85900000000004, 691.297 )
            , ( 426.568, 691.224 )
            , ( 430.625, 690.875 )
            , ( 434.712, 690.136 )
            , ( 438.51099999999997, 688.893 )
            , ( 441.704, 687.032 )
            , ( 443.97299999999996, 684.439 )
            , ( 445, 681 )
            , ( 446.41099999999994, 655.72 )
            , ( 447.64799999999997, 626.9599999999999 )
            , ( 448.71700000000004, 595.14 )
            , ( 449.624, 560.68 )
            , ( 450.375, 524 )
            , ( 450.976, 485.52000000000004 )
            , ( 451.433, 445.65999999999997 )
            , ( 451.752, 404.84000000000003 )
            , ( 451.939, 363.48 )
            , ( 452, 322 )
            , ( 451.719, 273.89000000000004 )
            , ( 450.952, 227.28000000000003 )
            , ( 449.813, 182.95 )
            , ( 448.416, 141.68 )
            , ( 446.875, 104.25 )
            , ( 445.304, 71.44000000000001 )
            , ( 443.817, 44.03 )
            , ( 442.528, 22.800000000000004 )
            , ( 441.55100000000004, 8.53 )
            , ( 441, 2 )
            , ( 446.617, 1.952 )
            , ( 454.616, 1.8559999999999999 )
            , ( 464.319, 1.784 )
            , ( 475.048, 1.808 )
            , ( 486.125, 2 )
            , ( 496.87199999999996, 2.432 )
            , ( 506.611, 3.1759999999999997 )
            , ( 514.664, 4.304 )
            , ( 520.3530000000001, 5.888 )
            , ( 523, 8 )
            , ( 522.922, 12.640999999999998 )
            , ( 523.216, 24.967999999999993 )
            , ( 523.774, 44.387000000000015 )
            , ( 524.488, 70.304 )
            , ( 525.25, 102.125 )
            , ( 525.9520000000001, 139.25599999999997 )
            , ( 526.486, 181.103 )
            , ( 526.744, 227.07199999999997 )
            , ( 526.6179999999999, 276.569 )
            , ( 526, 329 )
            , ( 525.3169999999999, 366.437 )
            , ( 524.496, 404.81600000000003 )
            , ( 523.579, 443.939 )
            , ( 522.6080000000001, 483.608 )
            , ( 521.625, 523.625 )
            , ( 520.6719999999999, 563.792 )
            , ( 519.7909999999999, 603.9110000000001 )
            , ( 519.024, 643.784 )
            , ( 518.413, 683.213 )
            , ( 518, 722 )
            , ( 514.004, 731.559 )
            , ( 503.07200000000006, 738.8320000000001 )
            , ( 486.78799999999995, 744.113 )
            , ( 466.736, 747.696 )
            , ( 444.5, 749.875 )
            , ( 421.664, 750.944 )
            , ( 399.812, 751.197 )
            , ( 380.528, 750.928 )
            , ( 365.396, 750.431 )
            ]
    in
    Polygon2d.singleLoop (List.map Point2d.fromCoordinates vertexCoordinates)
